@@ -27,7 +27,7 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 */
 #ifndef __CALASTMIX__
 #define __CALASTMIX__
-
+#ifndef ONLY_LOCAL_PROXY
 #include "CAMix.hpp"
 #include "CAMuxSocket.hpp"
 #include "CAASymCipher.hpp"
@@ -40,45 +40,117 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 #ifdef LOG_CRIME
 	#include "tre/regex.h"
 #endif
+#include "CALogPacketStats.hpp"
+#ifndef NEW_MIX_TYPE // not TypeB mixes
+  /* TypeB mixes are using an own implementation */
+  #include "CALastMixChannelList.hpp"
+#endif
 
-class CALastMix:public CAMix
+#ifdef REPLAY_DETECTION
+	#include "CAMixWithReplayDB.hpp"
+#endif
 
+THREAD_RETURN	lm_loopLog(void*);
+THREAD_RETURN	lm_loopSendToMix(void* param);
+THREAD_RETURN	lm_loopReadFromMix(void* pParam);
+	
+class CALastMix:public 
+#ifdef REPLAY_DETECTION
+	CAMixWithReplayDB
+#else
+	CAMix
+#endif
 	{
 		public:
 			CALastMix()
 				{
-					m_pMuxIn=NULL;m_pSignature=NULL;
-					m_pRSA=NULL;m_pInfoService=NULL;
+					m_pMuxIn=NULL;
+					m_pSignature=NULL;
+					m_pRSA=NULL;
+					m_pInfoService=NULL;
+          #ifndef NEW_MIX_TYPE // not TypeB mixes
+            /* TypeB mixes are using an own implementation */
+					m_pChannelList=NULL;
+          #endif
+					m_pthreadSendToMix=m_pthreadReadFromMix=NULL;
+					m_pQueueSendToMix=m_pQueueReadFromMix=NULL;
+					m_pCacheLB=new CACacheLoadBalancing();
+					m_pSocksLB=new CACacheLoadBalancing();
+					#ifdef LOG_PACKET_STATS
+						m_pLogPacketStats=NULL;
+					#endif
 				}
-			virtual ~CALastMix(){clean();}
-			SINT32 reconfigure();
-		protected:
-			virtual SINT32 loop();
-			SINT32 init();
-			SINT32 initOnce();
-			SINT32 clean();
 
-			SINT32 processKeyExchange();
+			virtual ~CALastMix()
+				{
+					clean();
+					delete m_pCacheLB;
+					m_pCacheLB = NULL;
+					delete m_pSocksLB;
+					m_pSocksLB = NULL;
+				}
+
+			SINT32 reconfigure();
+			tMixType getType() const
+				{
+					return CAMix::LAST_MIX;
+				}
+
+	protected:
+#ifdef DYNAMIC_MIX
+			void stopCascade()
+			{
+				m_bRestart = true;
+			}
+#endif
+			virtual SINT32 loop()=0;
+			SINT32 init();
+			virtual SINT32 initOnce();
+			SINT32 clean();
+      #ifdef NEW_MIX_TYPE // TypeB mixes
+        virtual void reconfigureMix();
+      #endif
+
+    // added by ronin <ronin2@web.de>
+    SINT32 initMixCascadeInfo(DOMElement* );
+
+    // moved to CAMix.hpp
+    virtual SINT32 processKeyExchange();
+    
+
 			SINT32 setTargets();
 #ifdef LOG_CRIME
-			bool	 checkCrime(UINT8* payLoad,UINT32 payLen);
+			bool	 checkCrime(const UINT8* payLoad,UINT32 payLen);
 #endif
 
 		protected:
+			volatile bool					m_bRestart;
 			CAMuxSocket*					m_pMuxIn;
-			CAQueue*							m_pQueueSendToMix;
-			CACacheLoadBalancing	m_oCacheLB;
-			CASocketAddrINet			maddrSocks;
+			CAQueue* m_pQueueSendToMix;
+			CAQueue* m_pQueueReadFromMix;
+			#ifdef LOG_PACKET_TIMES
+				CALogPacketStats* m_pLogPacketStats;
+			#endif
+			CACacheLoadBalancing*	m_pCacheLB;
+			CACacheLoadBalancing* m_pSocksLB;
 			CAASymCipher*					m_pRSA;
-			CASignature*					m_pSignature;
-			CAInfoService*				m_pInfoService;
-#ifdef LOG_CRIME
-			regex_t*							m_pCrimeRegExps;
-			UINT32								m_nCrimeRegExp;
-#endif
+			CAThread*							m_pthreadSendToMix;
+			CAThread*							m_pthreadReadFromMix;
+      #ifndef NEW_MIX_TYPE // not TypeB mixes
+        /* TypeB mixes are using an own implementation */
+			CALastMixChannelList* m_pChannelList;
+      #endif
 
+#ifdef LOG_CRIME
+			regex_t*							m_pCrimeRegExpsURL;
+			UINT32								m_nCrimeRegExpsURL;
+			regex_t*							m_pCrimeRegExpsPayload;
+			UINT32								m_nCrimeRegExpsPayload;
+#endif
+			
 		protected:
 			friend THREAD_RETURN	lm_loopSendToMix(void* param);
+			friend THREAD_RETURN	lm_loopReadFromMix(void* pParam);
 			friend THREAD_RETURN	lm_loopLog(void*);
 			volatile bool					m_bRunLog;
 			volatile UINT32				m_logUploadedPackets;
@@ -88,3 +160,4 @@ class CALastMix:public CAMix
 	};
 
 #endif
+#endif //ONLY_LOCAL_PROXY
