@@ -33,8 +33,10 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 #include "CACmdLnOptions.hpp"
 #include "CAUtil.hpp"
 #include "CASocketAddrINet.hpp"
+#include "CABase64.hpp"
+#include "xml/DOM_Output.hpp"
 #ifndef NEW_MIX_TYPE
-extern CACmdLnOptions options;
+extern CACmdLnOptions* pglobalOptions;
 // signals the main loop whether to capture or replay packets
 bool CALocalProxy::bCapturePackets;
 bool CALocalProxy::bReplayPackets;
@@ -58,9 +60,36 @@ void SIGUSR2_handler(int signum)
 	}
 
 
+SINT32 CALocalProxy::start()
+	{
+		if(initOnce()!=E_SUCCESS)
+			return E_UNKNOWN;
+    while(true)
+    {
+				CAMsg::printMsg(LOG_DEBUG, "CALocalProxy main: before init()\n");
+        if(init() == E_SUCCESS)
+        {
+					CAMsg::printMsg(LOG_DEBUG, "CALocalProxy main: init() returned success\n");
+          CAMsg::printMsg(LOG_INFO, "The local proxy is now on-line.\n");
+					loop();
+					CAMsg::printMsg(LOG_DEBUG, "CAMix local proxy: loop() returned, maybe connection lost.\n");
+        }
+        else
+        {
+            CAMsg::printMsg(LOG_DEBUG, "init() failed, maybe no connection.\n");
+        }
+
+				CAMsg::printMsg(LOG_DEBUG, "CALocalProxy main: before clean()\n");
+				clean();
+				CAMsg::printMsg(LOG_DEBUG, "CAMix LocalProxy: after clean()\n");
+        sSleep(20);
+    }
+}
+
+
 SINT32 CALocalProxy::initOnce()
 	{
-		if(options.getListenerInterfaceCount()<1)
+		if(pglobalOptions->getListenerInterfaceCount()<1)
 		  {
 				CAMsg::printMsg(LOG_CRIT,"No Listener Interface spezified!\n");
 				return E_UNKNOWN;
@@ -74,7 +103,7 @@ SINT32 CALocalProxy::initOnce()
 		#endif
 
 		UINT8 strTarget[255];
-		if(options.getMixHost(strTarget,255)!=E_SUCCESS)
+		if(pglobalOptions->getMixHost(strTarget,255)!=E_SUCCESS)
 		  {
 				CAMsg::printMsg(LOG_CRIT,"No AnonServer specified!\n");
 				return E_UNKNOWN;
@@ -88,7 +117,7 @@ SINT32 CALocalProxy::init()
 		
 		m_socketIn.create();
 		m_socketIn.setReuseAddr(true);
-		pListener=options.getListenerInterface(1);
+		pListener=pglobalOptions->getListenerInterface(1);
 		if(pListener==NULL)
 			{
 				CAMsg::printMsg(LOG_CRIT,"No listener specified\n");
@@ -105,9 +134,9 @@ SINT32 CALocalProxy::init()
 				return E_UNKNOWN;
 			}
 		delete pSocketAddrIn;
-/*		if(options.getSOCKSServerPort()!=(UINT16)-1)
+/*		if(pglobalOptions->getSOCKSServerPort()!=(UINT16)-1)
 			{
-				socketAddrIn.setAddr((UINT8*)"127.0.0.1",options.getSOCKSServerPort());
+				socketAddrIn.setAddr((UINT8*)"127.0.0.1",pglobalOptions->getSOCKSServerPort());
 				m_socketSOCKSIn.create();
 				m_socketSOCKSIn.setReuseAddr(true);
 				if(m_socketSOCKSIn.listen(socketAddrIn)!=E_SUCCESS)
@@ -118,27 +147,38 @@ SINT32 CALocalProxy::init()
 			}*/
 		CASocketAddrINet addrNext;
 		UINT8 strTarget[255];
-		options.getMixHost(strTarget,255);
-		addrNext.setAddr(strTarget,options.getMixPort());
+		pglobalOptions->getMixHost(strTarget,255);
+		addrNext.setAddr(strTarget,pglobalOptions->getMixPort());
 		CAMsg::printMsg(LOG_INFO,"Try connecting to next Mix...\n");
 
 		((CASocket*)m_muxOut)->create();
 		((CASocket*)m_muxOut)->setSendBuff(MIXPACKET_SIZE*50);
 		((CASocket*)m_muxOut)->setRecvBuff(MIXPACKET_SIZE*50);
 		if(m_muxOut.connect(addrNext)==E_SUCCESS)
-			{
-				
+			{				
 				CAMsg::printMsg(LOG_INFO," connected!\n");
 				UINT16 size;
-				((CASocket*)m_muxOut)->receive((UINT8*)&size,2);
-				((CASocket*)m_muxOut)->receive(&m_chainlen,1);
-				if(m_chainlen=='<')//assuming XML
+				UINT8 byte;
+				((CASocket*)m_muxOut)->receiveFully((UINT8*)&size,2);
+				((CASocket*)m_muxOut)->receiveFully((UINT8*)&byte,1);
+				CAMsg::printMsg(LOG_INFO,"Received Key Info!\n");
+				size=ntohs(size);
+#ifdef _DEBUG
+				CAMsg::printMsg(LOG_INFO,"Key Info size is:%u\n",size);
+#endif
+				if(byte=='<')//assuming XML
 					{
-						size=ntohs(size);
+#ifdef _DEBUG
+						CAMsg::printMsg(LOG_INFO,"Key Info is XML!\n");
+#endif
 						UINT8* buff=new UINT8[size+1];
-						buff[0]=m_chainlen;
+						buff[0]=byte;
 						((CASocket*)m_muxOut)->receiveFully(buff+1,size-1);
 						buff[size]=0;
+#ifdef _DEBUG
+						CAMsg::printMsg(LOG_INFO,"Key Info is:\n");
+						CAMsg::printMsg(LOG_INFO,"%s\n",buff);
+#endif
 						SINT32 ret=processKeyExchange(buff,size);
 						delete []  buff;
 						if(ret!=E_SUCCESS)
@@ -146,20 +186,7 @@ SINT32 CALocalProxy::init()
 					}
 				else
 					{
-						CAMsg::printMsg(LOG_INFO,"Chain-Length: %d\n",m_chainlen);
-						size=ntohs(size)-1;
-						UINT8* buff=new UINT8[size];
-						((CASocket*)m_muxOut)->receiveFully(buff,size);
-						m_arRSA=new CAASymCipher[m_chainlen];
-						int aktIndex=0;
-						for(int i=0;i<m_chainlen;i++)
-							{
-								int len=size;
-								m_arRSA[i].setPublicKey(buff+aktIndex,(UINT32*)&len);
-								size-=len;
-								aktIndex+=len;
-							}
-						delete[] buff;
+						return E_UNKNOWN;
 					}
 				return E_SUCCESS;
 			}
@@ -173,9 +200,9 @@ SINT32 CALocalProxy::init()
 SINT32 CALocalProxy::loop()
 	{
 		CASocketList  oSocketList;
-		CASocketGroup oSocketGroup;
+		CASocketGroup oSocketGroup(false);
 		oSocketGroup.add(m_socketIn);
-		UINT16 socksPort=options.getSOCKSServerPort();
+		UINT16 socksPort=pglobalOptions->getSOCKSServerPort();
 		bool bHaveSocks=(socksPort!=0xFFFF);
 		if(bHaveSocks)
 			oSocketGroup.add(m_socketSOCKSIn);
@@ -199,7 +226,7 @@ SINT32 CALocalProxy::loop()
 		for(;;)
 			{
 				// Add timeout to select to allow for a replay attack to take place.
-				if((countRead=oSocketGroup.select(false, 100))==SOCKET_ERROR)
+				if((countRead=oSocketGroup.select(100))==SOCKET_ERROR)
 					{
 						sSleep(1);
 						continue;
@@ -312,10 +339,10 @@ SINT32 CALocalProxy::loop()
 										}
 									else
 										{
-											for(int c=0;c<m_chainlen;c++)
-												oConnection.pCiphers[c].decryptAES2(pMixPacket->data,pMixPacket->data,DATA_SIZE);
+											for(UINT32 c=0;c<m_chainlen;c++)
+												oConnection.pCiphers[c].crypt2(pMixPacket->data,pMixPacket->data,DATA_SIZE);
 											#ifdef _DEBUG
-												CAMsg::printMsg(LOG_DEBUG,"Sending Data to Browser!");
+												CAMsg::printMsg(LOG_DEBUG,"Sending Data to Browser!\n");
 											#endif
 											oConnection.pSocket->send(pMixPacket->payload.data,ntohs(pMixPacket->payload.len));
 										}
@@ -331,13 +358,12 @@ SINT32 CALocalProxy::loop()
 									{
 										countRead--;
 										getRandom(pMixPacket->payload.data,PAYLOAD_SIZE);
-										if(!tmpCon->pCiphers[0].isEncyptionKeyValid())
-											len=tmpCon->pSocket->receive(pMixPacket->payload.data,PAYLOAD_SIZE-m_chainlen*(KEY_SIZE + TIMESTAMP_SIZE));
+										if(!tmpCon->pCiphers[0].isKeyValid())
+											len=tmpCon->pSocket->receive(pMixPacket->payload.data,PAYLOAD_SIZE-m_chainlen*KEY_SIZE);
 										else
 											len=tmpCon->pSocket->receive(pMixPacket->payload.data,PAYLOAD_SIZE);
 										if(len==SOCKET_ERROR||len==0)
 											{
-												//TODO delete cipher..
 												CASocket* tmpSocket=oSocketList.remove(tmpCon->outChannel);
 												if(tmpSocket!=NULL)
 													{
@@ -348,6 +374,7 @@ SINT32 CALocalProxy::loop()
 														m_muxOut.send(pMixPacket);
 														tmpSocket->close();
 														delete tmpSocket;
+														delete [] tmpCon->pCiphers;
 													}
 											}
 										else 
@@ -362,17 +389,17 @@ SINT32 CALocalProxy::loop()
 													{
 														pMixPacket->payload.type=MIX_PAYLOAD_HTTP;
 													}
-												if(!tmpCon->pCiphers[0].isEncyptionKeyValid()) //First time --> rsa key
+												if(!tmpCon->pCiphers[0].isKeyValid()) //First time --> rsa key
 													{
 														//Has to bee optimized!!!!
-														unsigned char buff[DATA_SIZE];
-														int size=DATA_SIZE-KEY_SIZE-TIMESTAMP_SIZE;
+														UINT8 buff[DATA_SIZE];
+														//UINT32 size=DATA_SIZE-KEY_SIZE;
 														//tmpCon->pCipher->generateEncryptionKey(); //generate Key
-														for(int c=0;c<m_chainlen;c++)
+														for(UINT32 c=0;c<m_chainlen;c++)
 															{
 																getRandom(buff,KEY_SIZE);
 																buff[0]&=0x7F; // Hack for RSA to ensure m < n !!!!!
-																tmpCon->pCiphers[c].setKeyAES(buff);
+																tmpCon->pCiphers[c].setKey(buff);
 																/*
 																	PROTOCOL CHANGE:
 																	This is a change in the protocol between JAP and the mixes:
@@ -383,22 +410,34 @@ SINT32 CALocalProxy::loop()
 																	packet (from 8 to 7).
 																*/
 																#ifdef WITH_TIMESTAMP
-																	currentTimestamp(buff+KEY_SIZE,true);
+																	//TODO insert timesampt
+																	//currentTimestamp(buff+KEY_SIZE,true);
 																#endif
-																memcpy(buff+KEY_SIZE+TIMESTAMP_SIZE,pMixPacket->data,size);
-																m_arRSA[c].encrypt(buff,buff);
-																// Does RSA_SIZE need to be increased by RSA_SIZE/KEY_SIZE*TIMESTAMP_SIZE?
-																tmpCon->pCiphers[c].encryptAES(buff+RSA_SIZE,buff+RSA_SIZE,DATA_SIZE-RSA_SIZE);
+																memcpy(buff+KEY_SIZE,pMixPacket->data,DATA_SIZE-KEY_SIZE);
+																if(m_MixCascadeProtocolVersion==MIX_CASCADE_PROTOCOL_VERSION_0_4&&c==m_chainlen-1)
+																	{
+																		m_pSymCipher->crypt1(buff,buff,KEY_SIZE);
+																		UINT8 iv[16];
+																		memset(iv,0xFF,16);
+																		tmpCon->pCiphers[c].setIV2(iv);
+																		tmpCon->pCiphers[c].crypt1(buff+KEY_SIZE,buff+KEY_SIZE,DATA_SIZE-KEY_SIZE);
+																	}
+																else
+																	{
+																		m_arRSA[c].encrypt(buff,buff);
+																		// Does RSA_SIZE need to be increased by RSA_SIZE/KEY_SIZE*TIMESTAMP_SIZE?
+																		tmpCon->pCiphers[c].crypt1(buff+RSA_SIZE,buff+RSA_SIZE,DATA_SIZE-RSA_SIZE);
+																	}	
 																memcpy(pMixPacket->data,buff,DATA_SIZE);
-																size-=KEY_SIZE+TIMESTAMP_SIZE;
-																len+=KEY_SIZE+TIMESTAMP_SIZE;
+																//size-=KEY_SIZE;
+																//len+=KEY_SIZE;
 															}
 														pMixPacket->flags=CHANNEL_OPEN;
 													}
 												else //sonst
 													{
-														for(int c=0;c<m_chainlen;c++)
-															tmpCon->pCiphers[c].encryptAES(pMixPacket->data,pMixPacket->data,DATA_SIZE);
+														for(UINT32 c=0;c<m_chainlen;c++)
+															tmpCon->pCiphers[c].crypt1(pMixPacket->data,pMixPacket->data,DATA_SIZE);
 														pMixPacket->flags=CHANNEL_DATA;
 													}
 
@@ -447,7 +486,7 @@ MIX_CONNECTION_ERROR:
 		delete pMixPacket;
 		if(ret==E_SUCCESS)
 			return E_SUCCESS;
-		if(options.getAutoReconnect())
+		if(pglobalOptions->getAutoReconnect())
 			return E_UNKNOWN;
 		else
 			exit(-1);
@@ -461,11 +500,16 @@ SINT32 CALocalProxy::clean()
 		if(m_arRSA!=NULL)
 			delete[] m_arRSA;
 		m_arRSA=NULL;
+		if(m_pSymCipher!=NULL)
+			delete m_pSymCipher;
+		m_pSymCipher=NULL;
 		return E_SUCCESS;
 	}
 
 SINT32 CALocalProxy::processKeyExchange(UINT8* buff,UINT32 len)
 	{
+		CAMsg::printMsg(LOG_INFO,"Login process and key exchange started...\n");
+#ifndef ONLY_LOCAL_PROXY
 		//Parsing KeyInfo received from Mix n+1
 		MemBufInputSource oInput(buff,len,"localoproxy");
 		DOMParser oParser;
@@ -477,43 +521,179 @@ SINT32 CALocalProxy::processKeyExchange(UINT8* buff,UINT32 len)
 				return E_UNKNOWN;
 			}
 
+
 		DOM_Element root=doc.getDocumentElement();
+		DOM_Element elemVersion;
+		getDOMChildByName(root,(UINT8*)"MixProtocolVersion",elemVersion,false);
+		UINT8 strVersion[255];
+		UINT32 tmpLen=255;
+		if(getDOMElementValue(elemVersion,strVersion,&tmpLen)==E_SUCCESS&&tmpLen==3)
+			{
+#ifdef _DEBUG
+				CAMsg::printMsg(LOG_INFO,"XML MixProtocolVersion value:%s\n",strVersion);
+#endif
+				if(memcmp(strVersion,"0.4",3)==0)
+					{
+						CAMsg::printMsg(LOG_INFO,"MixCascadeProtocolVersion: 0.4\n");
+						m_MixCascadeProtocolVersion=MIX_CASCADE_PROTOCOL_VERSION_0_4;
+					}	
+				else if(memcmp(strVersion,"0.3",3)==0)
+					{
+						CAMsg::printMsg(LOG_INFO,"MixCascadeProtocolVersion: 0.3\n");
+						m_MixCascadeProtocolVersion=MIX_CASCADE_PROTOCOL_VERSION_0_3;
+					}
+				else if(memcmp(strVersion,"0.2",3)==0)
+					{
+						CAMsg::printMsg(LOG_INFO,"MixCascadeProtocolVersion: 0.2\n");
+						m_MixCascadeProtocolVersion=MIX_CASCADE_PROTOCOL_VERSION_0_2;
+					}
+				else
+					{
+						CAMsg::printMsg(LOG_ERR,"Unsupported MixProtocolVersion!\n");
+						return E_UNKNOWN;
+					}
+			}
+		else
+		{
+#ifdef _DEBUG
+			CAMsg::printMsg(LOG_ERR,"No MixProtocolVersion given!\n");
+			return E_UNKNOWN;
+#endif
+		}
 		DOM_Element elemMixes;
 		getDOMChildByName(root,(UINT8*)"Mixes",elemMixes,false);
-		int chainlen=-1;
+		SINT32 chainlen=-1;
 		if(elemMixes==NULL||getDOMElementAttribute(elemMixes,"count",&chainlen)!=E_SUCCESS)
-			return E_UNKNOWN;
+			{
+#ifdef _DEBUG
+				CAMsg::printMsg(LOG_ERR,"No count of Mixes given!\n");
+				return E_UNKNOWN;
+#endif
+			}
 		m_chainlen=(UINT32)chainlen;
+#ifdef _DEBUG
+		CAMsg::printMsg(LOG_INFO,"Number of Mixes is: %u\n",m_chainlen);
+#endif
+		if(m_chainlen==0)
+			{
+#ifdef _DEBUG
+				CAMsg::printMsg(LOG_ERR,"Number of Mixes is 0!\n");
+				return E_UNKNOWN;
+#endif
+			}
 		UINT32 i=0;
 		m_arRSA=new CAASymCipher[m_chainlen];
 		DOM_Node child=elemMixes.getLastChild();
-		while(child!=NULL&&chainlen>0)
+		while(child!= NULL&&chainlen>0)
 			{
 				if(child.getNodeName().equals("Mix"))
 					{
 						DOM_Node nodeKey=child.getFirstChild();
 						if(m_arRSA[i++].setPublicKeyAsDOMNode(nodeKey)!=E_SUCCESS)
-							return E_UNKNOWN;						
+							{
+#ifdef _DEBUG
+								CAMsg::printMsg(LOG_ERR,"Error in parsing the public key of a Mix\n");
+								return E_UNKNOWN;
+#endif
+							}
 						chainlen--;
 					}
 				child=child.getPreviousSibling();
 			}
 		if(chainlen!=0)
-			return E_UNKNOWN;
+			{
+#ifdef _DEBUG
+				CAMsg::printMsg(LOG_ERR,"Expected information for %u Mixes but found only %u!\n",m_chainlen,m_chainlen-chainlen);
+				return E_UNKNOWN;
+#endif
+			}
+#else
+	m_MixCascadeProtocolVersion=MIX_CASCADE_PROTOCOL_VERSION_0_4;
+	m_chainlen=2;
+	m_arRSA=new CAASymCipher[m_chainlen];
+	UINT8* modulus;
+	UINT32 moduluslen;
+	UINT8* exponent;
+	UINT32 exponentlen;
+	modulus=(UINT8*)strstr((char*)buff,"<Modulus>")+9;
+	moduluslen=((UINT8*)strstr((char*)modulus,"</Modulus>"))-modulus;
+	exponent=(UINT8*)strstr((char*)modulus,"<Exponent>")+10;
+	exponentlen=((UINT8*)strstr((char*)exponent,"</Exponent>"))-exponent;
+	m_arRSA[1].setPublicKey(modulus,moduluslen,exponent,exponentlen);
+	modulus=(UINT8*)strstr((char*)exponent,"<Modulus>")+9;
+	moduluslen=((UINT8*)strstr((char*)modulus,"</Modulus>"))-modulus;
+	exponent=(UINT8*)strstr((char*)modulus,"<Exponent>")+10;
+	exponentlen=((UINT8*)strstr((char*)exponent,"</Exponent>"))-exponent;
+	m_arRSA[0].setPublicKey(modulus,moduluslen,exponent,exponentlen);
+#endif
 		//Now sending SymKeys....
-		MIXPACKET oPacket;
-		oPacket.flags=0;
-		oPacket.channel=0;
-		UINT8 keys[32];
-		getRandom(keys,32);
-		m_muxOut.setReceiveKey(keys,16);
-		m_muxOut.setSendKey(keys+16,16);
-		getRandom(oPacket.data,DATA_SIZE);
-		memcpy(oPacket.data,"KEYPACKET",9);
-		memcpy(oPacket.data+9,keys,32);
-		m_arRSA[m_chainlen-1].encrypt(oPacket.data,oPacket.data);
-		m_muxOut.send(&oPacket);
-		m_muxOut.setCrypt(true);
+		if(m_MixCascadeProtocolVersion==MIX_CASCADE_PROTOCOL_VERSION_0_2)
+			{
+				MIXPACKET oPacket;
+				getRandom((UINT8*)&oPacket,MIXPACKET_SIZE);
+				oPacket.flags=0;
+				oPacket.channel=0;
+				UINT8 keys[32];
+				getRandom(keys,32);
+				m_muxOut.setReceiveKey(keys,16);
+				m_muxOut.setSendKey(keys+16,16);
+				memcpy(oPacket.data,"KEYPACKET",9);
+				memcpy(oPacket.data+9,keys,32);
+				m_arRSA[m_chainlen-1].encrypt(oPacket.data,oPacket.data);
+				m_muxOut.send(&oPacket);
+				m_muxOut.setCrypt(true);
+			}
+		else
+			{
+				const char* XML_HEADER="<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
+				const UINT32 XML_HEADER_SIZE=strlen(XML_HEADER);
+				const char* XML_JAP_KEY_TEMPLATE="<JAPKeyExchange version=\"0.1\"><LinkEncryption>%s</LinkEncryption><MixEncryption>%s</MixEncryption></JAPKeyExchange>";
+	      //DOM_Document doc=DOM_Document::createDocument();
+   			//DOM_Element e = doc.createElement("JAPKeyExchange");
+				//doc.appendChild(e);
+				//e.setAttribute("version", "0.1");
+				//DOM_Element elemLinkEnc = doc.createElement("LinkEncryption");
+				UINT8* buff=new UINT8[9000];
+				UINT8 linkKeys[64];
+				getRandom(linkKeys,64);
+				UINT8 outBuffLinkKey[512];
+				UINT32 outlenLinkKey=512;
+				CABase64::encode(linkKeys,64,outBuffLinkKey,&outlenLinkKey);
+				outBuffLinkKey[outlenLinkKey]=0;
+				//setDOMElementValue(elemLinkEnc,outBuff);
+				//e.appendChild(elemLinkEnc);
+				//DOM_Element elemMixEnc = doc.createElement("MixEncryption");
+				UINT8 mixKeys[32];
+				getRandom(mixKeys,32);
+				m_pSymCipher=new CASymCipher();
+				m_pSymCipher->setKey(mixKeys);
+				m_pSymCipher->setIVs(mixKeys+16);
+				UINT8 outBuffMixKey[512];
+				UINT32 outlenMixKey=512;
+				CABase64::encode(mixKeys,32,outBuffMixKey,&outlenMixKey);
+				outBuffMixKey[outlenMixKey]=0;
+				//setDOMElementValue(elemMixEnc,outBuff);
+				//e.appendChild(elemMixEnc);
+				sprintf((char*)buff,XML_JAP_KEY_TEMPLATE,outBuffLinkKey,outBuffMixKey);
+				UINT32 encbufflen;
+				UINT8* encbuff=encryptXMLElement(buff,strlen((char*)buff),encbufflen,&m_arRSA[m_chainlen-1]);
+				UINT16 size2=htons(encbufflen+XML_HEADER_SIZE);
+				SINT32 ret=((CASocket*)&m_muxOut)->send((UINT8*)&size2,2);
+				ret=((CASocket*)&m_muxOut)->send((UINT8*)XML_HEADER,XML_HEADER_SIZE);
+				ret=((CASocket*)&m_muxOut)->send(encbuff,encbufflen);
+				delete[] encbuff;
+				delete[] buff;
+				// Checking Signature send from Mix
+				ret=((CASocket*)&m_muxOut)->receiveFully((UINT8*)&size2,2);
+				size2=ntohs(size2);
+				UINT8* xmlbuff=new UINT8[size2];
+				ret=((CASocket*)&m_muxOut)->receiveFully(xmlbuff,size2);
+				delete[] xmlbuff;
+				m_muxOut.setSendKey(linkKeys,32);
+				m_muxOut.setReceiveKey(linkKeys+32,32);
+				m_muxOut.setCrypt(true);
+			}
+		CAMsg::printMsg(LOG_INFO,"Login process and key exchange finished!\n");		
 		return E_SUCCESS;
 	}
 #endif //!NEW_MIX_TYPE
