@@ -35,18 +35,23 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 // How many packets do you want to replay at a time?
 #define REPLAY_COUNT 16
 #ifndef NEW_MIX_TYPE
-class CALocalProxy:public CAMix
+class CALocalProxy
 	{
 		public:
-			CALocalProxy(){m_arRSA=NULL;}
-			virtual ~CALocalProxy(){clean();}
-
+			CALocalProxy()
+				{
+					m_arRSA=NULL;
+					m_pSymCipher=NULL;
+				}
+			
+			~CALocalProxy(){clean();}
 			
 			// signals the main loop whether to capture or replay packets
 			static bool bCapturePackets;
 			static bool bReplayPackets;
 			static int iCapturedPackets;
-
+			SINT32 start();
+	
 		private:
 			SINT32 loop();
 			SINT32 init();
@@ -57,8 +62,10 @@ class CALocalProxy:public CAMix
 			CASocket m_socketIn;
 			CASocket m_socketSOCKSIn;
 			CAMuxSocket m_muxOut;
-			UINT8 m_chainlen;
+			UINT32 m_chainlen;
+			UINT32 m_MixCascadeProtocolVersion;
 			CAASymCipher* m_arRSA;
+			CASymCipher* m_pSymCipher;
 	};
 #endif //!NEW_MIX_TYPE
 #endif
