@@ -34,32 +34,84 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 #include "CAMiddleMixChannelList.hpp"
 #include "CASignature.hpp"
 #include "CAInfoService.hpp"
+#include "CAMixWithReplayDB.hpp"
 
-class CAMiddleMix:public CAMix
-	{
+class CAMiddleMix:public
+#ifdef REPLAY_DETECTION
+	CAMixWithReplayDB
+#else
+	CAMix
+#endif
+{
 		public:
 			CAMiddleMix()
 				{
 					m_pMiddleMixChannelList=NULL;
 					m_pMuxOut=NULL;m_pMuxIn=NULL;m_pRSA=NULL;m_pSignature=NULL;
 					m_pInfoService=NULL;
+					m_pQueueSendToMixBefore=m_pQueueSendToMixAfter=NULL;
+#ifdef DYNAMIC_MIX
+					m_bBreakNeeded = false;
+#endif
 				}
 			virtual ~CAMiddleMix(){clean();};
+			tMixType getType() const
+				{
+					return CAMix::MIDDLE_MIX;
+				}
+		friend THREAD_RETURN mm_loopSendToMixBefore(void*);
+		friend THREAD_RETURN mm_loopSendToMixAfter(void*);
+		friend THREAD_RETURN mm_loopReadFromMixBefore(void*);
+		friend THREAD_RETURN mm_loopReadFromMixAfter(void*);
+		friend THREAD_RETURN mm_loopDownStream(void *);
+
 		private:
 			SINT32 loop();
 			SINT32 init();
 			SINT32 initOnce();
 			SINT32 clean();
+			SINT32 connectToNextMix(CASocketAddr* a_pAddrNext);
+#ifdef DYNAMIC_MIX
+			bool m_bBreakNeeded;
+#endif
 			
-			SINT32 proccessKeyExchange();
-		private:
+    /**
+    * This method is not applicable to middle mixes; it does nothing
+    * @param d ignored
+    * @retval E_SUCCESS in any case
+    */
+    virtual SINT32 initMixCascadeInfo(DOMElement* & d)
+    {
+        return E_SUCCESS;
+    }
+    
+    virtual SINT32 processKeyExchange();
+
+private:
 			CAMuxSocket* m_pMuxIn;
 			CAMuxSocket* m_pMuxOut;
 			CAASymCipher* m_pRSA;
-			CASignature* m_pSignature;
+
+			//CASignature* m_pSignature;
+			volatile bool m_bRun;
 			CAMiddleMixChannelList* m_pMiddleMixChannelList;
-			CAInfoService* m_pInfoService;
-			friend THREAD_RETURN mm_loopDownStream(void *p);
+			//CAInfoService* m_pInfoService;
+//			friend THREAD_RETURN mm_loopDownStream(void *p);
+protected:
+			CAQueue* m_pQueueSendToMixBefore;
+			CAQueue* m_pQueueSendToMixAfter;
+
+			UINT32 m_u32KeepAliveRecvInterval2;
+			UINT32 m_u32KeepAliveSendInterval2;
+
+#ifdef DYNAMIC_MIX
+			void stopCascade()
+			{
+				m_bRun = false;
+			}
+#endif
 	};
 
+
+
 #endif
