@@ -40,6 +40,12 @@ CAMuxSocket::CAMuxSocket()
 		m_aktBuffPos=0;
 		m_bIsCrypted=false;
 	}
+	
+CAMuxSocket::~CAMuxSocket()
+	{
+		close();
+		delete []m_Buff;
+	}	
 
 SINT32 CAMuxSocket::setCrypt(bool b)
 	{
@@ -62,22 +68,29 @@ SINT32 CAMuxSocket::accept(UINT16 port)
 		if(oSocket.accept(m_Socket)!=E_SUCCESS)
 			return E_UNKNOWN;
 		oSocket.close();
-		m_Socket.setRecvLowWat(MIXPACKET_SIZE);
+		//m_Socket.setRecvLowWat(MIXPACKET_SIZE);
 		m_aktBuffPos=0;
 		return E_SUCCESS;
 	}
 
+/** Waits for an incoming connection on oAddr.
+  * @retval E_SUCCESS, if successful
+	* @retval E_SOCKET_BIND, E_SOCKET_LISTEN
+	* @retval E_UNKOWN
+	*/
 SINT32 CAMuxSocket::accept(const CASocketAddr& oAddr)
 	{
 		CASocket oSocket;
 		oSocket.create(oAddr.getType());
 		oSocket.setReuseAddr(true);
-		if(oSocket.listen(oAddr)!=E_SUCCESS)
-			return E_UNKNOWN;
-		if(oSocket.accept(m_Socket)!=E_SUCCESS)
+		SINT32 ret=oSocket.listen(oAddr);
+		if(ret!=E_SUCCESS)
+			return ret;
+		ret=oSocket.accept(m_Socket);
+		if(ret!=E_SUCCESS)
 			return E_UNKNOWN;
 		oSocket.close();
-		m_Socket.setRecvLowWat(MIXPACKET_SIZE);
+		//m_Socket.setRecvLowWat(MIXPACKET_SIZE);
 		m_aktBuffPos=0;
 		return E_SUCCESS;
 	}
@@ -89,7 +102,7 @@ SINT32 CAMuxSocket::connect(CASocketAddr & psa)
 
 SINT32 CAMuxSocket::connect(CASocketAddr & psa,UINT retry,UINT32 time)
 	{
-		m_Socket.setRecvLowWat(MIXPACKET_SIZE);
+		//m_Socket.setRecvLowWat(MIXPACKET_SIZE);
 		m_aktBuffPos=0;
 		return m_Socket.connect(psa,retry,time);
 	}
@@ -112,11 +125,9 @@ SINT32 CAMuxSocket::send(MIXPACKET *pPacket)
 		UINT8 tmpBuff[16];
 		memcpy(tmpBuff,pPacket,16);
 		pPacket->channel=htonl(pPacket->channel);
-#ifndef NEW_MIX_TYPE
 		pPacket->flags=htons(pPacket->flags);
-#endif
 		if(m_bIsCrypted)
-    	m_oCipherOut.encryptAES(((UINT8*)pPacket),((UINT8*)pPacket),16);
+    	m_oCipherOut.crypt1(((UINT8*)pPacket),((UINT8*)pPacket),16);
 		ret=m_Socket.sendFully(((UINT8*)pPacket),MIXPACKET_SIZE);
 		if(ret!=E_SUCCESS)
 			{
@@ -140,18 +151,33 @@ SINT32 CAMuxSocket::send(MIXPACKET *pPacket,UINT8* buff)
 		UINT8 tmpBuff[16];
 		memcpy(tmpBuff,pPacket,16);
 		pPacket->channel=htonl(pPacket->channel);
-#ifndef NEW_MIX_TYPE
 		pPacket->flags=htons(pPacket->flags);
-#endif
 		if(m_bIsCrypted)
-			m_oCipherOut.encryptAES(((UINT8*)pPacket),((UINT8*)pPacket),16);
+			m_oCipherOut.crypt1(((UINT8*)pPacket),((UINT8*)pPacket),16);
 		memcpy(buff,((UINT8*)pPacket),MIXPACKET_SIZE);
 		ret=MIXPACKET_SIZE;
 		memcpy(pPacket,tmpBuff,16);
 		m_csSend.unlock();
 		return ret;
 	}
-
+	
+SINT32 CAMuxSocket::prepareForSend(MIXPACKET *pinoutPacket)
+	{	
+		m_csSend.lock();
+		pinoutPacket->channel=htonl(pinoutPacket->channel);
+		pinoutPacket->flags=htons(pinoutPacket->flags);
+		m_oCipherOut.crypt1(((UINT8*)pinoutPacket),((UINT8*)pinoutPacket),16);
+		m_csSend.unlock();
+		return MIXPACKET_SIZE;
+	}
+	
+/** Receives a whole MixPacket. Blocks until a packet is received or
+	* a socket error occurs.
+	*
+	* @param pPacket on return stores the received MixPacket
+	* @retval SOCKET_ERROR, in case of an error
+	* @retval MIXPACKET_SIZE otherwise
+	*/
 SINT32 CAMuxSocket::receive(MIXPACKET* pPacket)
 	{
 		m_csReceive.lock();
@@ -161,21 +187,21 @@ SINT32 CAMuxSocket::receive(MIXPACKET* pPacket)
 				return SOCKET_ERROR;
 			}
 		if(m_bIsCrypted)
-    	m_oCipherIn.decryptAES((UINT8*)pPacket,(UINT8*)pPacket,16);
+    	m_oCipherIn.crypt1((UINT8*)pPacket,(UINT8*)pPacket,16);
 		pPacket->channel=ntohl(pPacket->channel);
-#ifndef NEW_MIX_TYPE
 		pPacket->flags=ntohs(pPacket->flags);
-#endif
 		m_csReceive.unlock();
 		return MIXPACKET_SIZE;
 	}
 
-/**Trys to receive a Mix-Packet. If after timout milliseconds not a whole packet is available
-* E_AGAIN will be returned. In this case you should later try to get the rest of the packet
+/**Trys to receive a Mix-Packet. If after timout milliseconds not 
+* a whole packet is available
+* E_AGAIN will be returned. 
+* In this case you should later try to get the rest of the packet
 */
 
-//TODO: Bug if socket is in non_blocking mode!!
-SINT32 CAMuxSocket::receive(MIXPACKET* pPacket,UINT32 timeout)
+//TODO: Bug if socket is not in non_blocking mode!!
+SINT32 CAMuxSocket::receive(MIXPACKET* pPacket,UINT32 msTimeout)
 	{
 		m_csReceive.lock();
 		SINT32 len=MIXPACKET_SIZE-m_aktBuffPos;
@@ -188,35 +214,37 @@ SINT32 CAMuxSocket::receive(MIXPACKET* pPacket,UINT32 timeout)
 		if(ret==len) //whole packet recieved
 			{
 				if(m_bIsCrypted)
-					m_oCipherIn.decryptAES(m_Buff,m_Buff,16);
+					m_oCipherIn.crypt1(m_Buff,m_Buff,16);
 				memcpy(pPacket,m_Buff,MIXPACKET_SIZE);
 				pPacket->channel=ntohl(pPacket->channel);
-#ifndef NEW_MIX_TYPE
 				pPacket->flags=ntohs(pPacket->flags);
-#endif
 				m_aktBuffPos=0;
 				m_csReceive.unlock();
 				return MIXPACKET_SIZE;
 			}
 		if(ret>0) //some new bytes arrived
 			m_aktBuffPos+=ret;
-		if(timeout==0) //we should not wait any more
+		if(msTimeout==0) //we should not wait any more
 			{
 				m_csReceive.unlock();
 				return E_AGAIN;
 			}
-		UINT32 timeE=time(NULL)+timeout;
-		SINT32 dt=timeout;
-		CASingleSocketGroup oSocketGroup;
+		UINT64 timeE;
+		UINT64 timeC;
+		getcurrentTimeMillis(timeE);
+		add64(timeE,msTimeout);
+		UINT32 dt=msTimeout;
+		CASingleSocketGroup oSocketGroup(false);
 		oSocketGroup.add(*this);
-		do
+		for(;;)
 			{
-				ret=oSocketGroup.select(false,dt);
+				ret=oSocketGroup.select(dt);
 				if(ret!=1)
 					{
 						m_csReceive.unlock();
 						return E_UNKNOWN;
 					}
+				len=MIXPACKET_SIZE-m_aktBuffPos;
 				ret=m_Socket.receive(m_Buff+m_aktBuffPos,len);
 				if(ret<=0&&ret!=E_AGAIN)
 					{
@@ -226,20 +254,21 @@ SINT32 CAMuxSocket::receive(MIXPACKET* pPacket,UINT32 timeout)
 				if(ret==len)
 					{
 						if(m_bIsCrypted)
-							m_oCipherIn.decryptAES(m_Buff,m_Buff,16);
+							m_oCipherIn.crypt1(m_Buff,m_Buff,16);
 						memcpy(pPacket,m_Buff,MIXPACKET_SIZE);
 						pPacket->channel=ntohl(pPacket->channel);
-#ifndef NEW_MIX_TYPE
 						pPacket->flags=ntohs(pPacket->flags);
-#endif
 						m_aktBuffPos=0;
 						m_csReceive.unlock();
 						return MIXPACKET_SIZE;
 					}
 				if(ret>0)
 					m_aktBuffPos+=ret;
-				dt=timeE-time(NULL);
-			}	while(dt>0);
+				getcurrentTimeMillis(timeC);
+				if(isGreater64(timeC,timeE)||isEqual64(timeC,timeE))
+					break;
+				dt=diff64(timeE,timeC);
+			}
 		m_csReceive.unlock();
 		return E_AGAIN;
 	}
@@ -261,15 +290,14 @@ int CAMuxSocket::close(HCHANNEL channel_id,UINT8* buff)
 	}*/
 
 #ifdef LOG_CRIME
-UINT32 CAMuxSocket::sigCrime(HCHANNEL channel_id,UINT8* buff)
+UINT32 CAMuxSocket::sigCrime(HCHANNEL channel_id,MIXPACKET* sigPacket)
 	{
-		MIXPACKET oPacket;
-		oPacket.channel=channel_id;
+		sigPacket->channel=channel_id;
 		UINT32 v;
 		getRandom(&v);
 		v&=CHANNEL_SIG_CRIME_ID_MASK;
-		oPacket.flags=(CHANNEL_SIG_CRIME|v);
-		send(&oPacket,buff);
+		sigPacket->flags=(CHANNEL_SIG_CRIME|v);
+		getRandom(sigPacket->data,DATA_SIZE);
 		return (v>>8);
 	}
 #endif
