@@ -31,130 +31,13 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 #include "CASocket.hpp"
 #include "CASymCipher.hpp"
 #include "CAMutex.hpp"
-
-typedef UINT32 HCHANNEL;
-#define MIX_PAYLOAD_HTTP  0 
-#define MIX_PAYLOAD_SOCKS 1
-
-
-#define MIXPACKET_SIZE 	998
-
-#define CHANNEL_DATA		0x00
-#define CHANNEL_OPEN		0x08 //must be 8 in final version!!
-
-#define CHANNEL_CLOSE		0x01
-#define CHANNEL_SUSPEND 0x02
-#define	CHANNEL_RESUME	0x04
-#define	CHANNEL_DUMMY		0x10
-
-#ifdef LOG_CRIME
-	#define	CHANNEL_SIG_CRIME 0x20
-	#define	CHANNEL_SIG_CRIME_ID_MASK 0x0000FF00
-	#define CHANNEL_ALLOWED_FLAGS		(CHANNEL_OPEN|CHANNEL_CLOSE|CHANNEL_SUSPEND|CHANNEL_RESUME|CHANNEL_SIG_CRIME|CHANNEL_SIG_CRIME_ID_MASK)
-#else
-	#define CHANNEL_ALLOWED_FLAGS		(CHANNEL_OPEN|CHANNEL_CLOSE|CHANNEL_SUSPEND|CHANNEL_RESUME)
-#endif
-
-#ifndef NEW_MIX_TYPE
-
-	#define DATA_SIZE 			992
-	#define PAYLOAD_SIZE 		989
-
-	#if defined(WIN32) ||defined(__sgi)
-		#pragma pack( push, t_MixPacket )
-		#pragma pack(1)
-		struct t_MixPacket
-			{
-				HCHANNEL channel;
-				UINT16  flags;
-				union
-					{
-						UINT8		data[DATA_SIZE];
-						struct t_MixPacketPayload
-							{
-								UINT16 len;
-								UINT8 type;
-								UINT8 data[PAYLOAD_SIZE];
-						} payload;
-					};
-			};
-		#pragma pack( pop, t_MixPacket )
-	#else
-		struct t_MixPacketPayload
-			{
-				UINT16 len;
-				UINT8 type;
-				UINT8 data[PAYLOAD_SIZE];
-			} __attribute__ ((__packed__));
-
-		struct t_MixPacket
-			{
-				HCHANNEL channel;
-				UINT16  flags;
-				union
-					{
-						UINT8		data[DATA_SIZE];
-						struct t_MixPacketPayload payload;
-					};
-			} __attribute__ ((__packed__)); // MUXPACKET __attribute__ ((__packed__));
-	#endif //WIN32 
-#else //NEW_MIX_TYPE
-
-	#define DATA_SIZE 			994
-	#define PAYLOAD_SIZE 		979
-
-	#if defined(WIN32) ||defined(__sgi)
-		#pragma pack( push, t_MixPacket )
-		#pragma pack(1)
-		struct t_MixPacket
-			{
-				HCHANNEL channel;
-				union
-					{
-						UINT8		data[DATA_SIZE];
-						struct t_MixPacketPayload
-							{
-								UINT8 linkid[8];
-								UINT8 seq[4];
-								UINT8 type;
-								UINT16 len;
-								UINT8 data[PAYLOAD_SIZE];
-						} payload;
-					};
-			};
-		#pragma pack( pop, t_MixPacket )
-	#else
-		struct t_MixPacketPayload
-			{
-				UINT8 linkid[8];
-				UINT8 seq[4];
-				UINT8 type;
-				UINT16 len;
-				UINT8 data[PAYLOAD_SIZE];
-			} __attribute__ ((__packed__));
-
-		struct t_MixPacket
-			{
-				HCHANNEL channel;
-				union
-					{
-						UINT8		data[DATA_SIZE];
-						struct t_MixPacketPayload payload;
-					};
-			} __attribute__ ((__packed__)); // MUXPACKET __attribute__ ((__packed__));
-	#endif //WIN32 
-#endif //NEW_MIX_TYPE
-
-typedef t_MixPacket MIXPACKET;
+#include "CATLSClientSocket.hpp"
 
 class CAMuxSocket
 	{
 		public:
 			CAMuxSocket();
-			~CAMuxSocket()
-				{
-					delete []m_Buff;
-				}
+			~CAMuxSocket();
 			SINT32 accept(UINT16 port);
 			SINT32 accept(const CASocketAddr& oAddr);
 			SINT32 connect(CASocketAddr& psa);
@@ -162,12 +45,19 @@ class CAMuxSocket
 			SINT32 close();
 			SINT32 send(MIXPACKET *pPacket);
 			SINT32 send(MIXPACKET *pPacket,UINT8* buff);
+			SINT32 prepareForSend(MIXPACKET* inoutPacket);
 			SINT32 receive(MIXPACKET *pPacket);
 			SINT32 receive(MIXPACKET *pPacket,UINT32 timeout);
+			
+			/** Receives some "plain" bytes from the underlying socket - just a convinient function...*/
+			SINT32 receiveFully(UINT8* buff,UINT32 len)
+				{
+					return m_Socket.receiveFully(buff,len);
+				}
 			//int close(HCHANNEL channel_id);
 			//int close(HCHANNEL channel_id,UINT8* buff);
 #ifdef LOG_CRIME
-			UINT32 sigCrime(HCHANNEL channel_id,UINT8* buff);
+			UINT32 sigCrime(HCHANNEL channel_id,MIXPACKET* sigPacket);
 #endif
 			operator CASocket*(){return &m_Socket;}
 			operator SOCKET(){return (SOCKET)m_Socket;}
@@ -192,13 +82,13 @@ class CAMuxSocket
 				{
 					if(keyLen==16)
 						{
-							m_oCipherIn.setKeyAES(key);
-							m_oCipherOut.setKeyAES(key);
+							m_oCipherIn.setKey(key);
+							m_oCipherOut.setKey(key);
 						}
 					else if(keyLen==32)
 						{
-							m_oCipherOut.setKeyAES(key);
-							m_oCipherIn.setKeyAES(key+16);
+							m_oCipherOut.setKey(key);
+							m_oCipherIn.setKey(key+16);
 						}
 				else
 						return E_UNKNOWN;
@@ -209,12 +99,12 @@ class CAMuxSocket
 				{
 					if(keyLen==16)
 						{
-							m_oCipherOut.setKeyAES(key);
+							m_oCipherOut.setKey(key);
 						}
 					else if(keyLen==32)
 						{
-							m_oCipherOut.setKeyAES(key);
-							m_oCipherOut.setIV(key+16);
+							m_oCipherOut.setKey(key);
+							m_oCipherOut.setIVs(key+16);
 						}
 					else
 						return E_UNKNOWN;
@@ -225,12 +115,12 @@ class CAMuxSocket
 				{
 					if(keyLen==16)
 						{
-							m_oCipherIn.setKeyAES(key);
+							m_oCipherIn.setKey(key);
 						}
 					else if(keyLen==32)
 						{
-							m_oCipherIn.setKeyAES(key);
-							m_oCipherIn.setIV(key+16);
+							m_oCipherIn.setKey(key);
+							m_oCipherIn.setIVs(key+16);
 						}
 					else
 						return E_UNKNOWN;
@@ -241,9 +131,9 @@ class CAMuxSocket
 				CASocket		m_Socket;
 				UINT32			m_aktBuffPos;
 				UINT8*			m_Buff;
-				CASymCipher m_oCipherIn;
-				CASymCipher m_oCipherOut;
-				bool				m_bIsCrypted;
+				CASymCipher		m_oCipherIn;
+				CASymCipher		m_oCipherOut;
+				bool			m_bIsCrypted;
 				CAMutex			m_csSend;
 				CAMutex			m_csReceive;
 	};
