@@ -26,6 +26,7 @@ IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISI
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
 #include "StdAfx.h"
+#ifndef ONLY_LOCAL_PROXY
 #include "CASignature.hpp"
 #include "CABase64.hpp"
 #include "CAUtil.hpp"
@@ -48,9 +49,7 @@ CASignature* CASignature::clone()
 		CASignature* tmpSig=new CASignature();
 		if(m_pDSA!=NULL)
 			{
-				DSA* tmpDSA=DSAparams_dup(m_pDSA);
-				tmpDSA->priv_key=BN_dup(m_pDSA->priv_key);
-				tmpDSA->pub_key=BN_dup(m_pDSA->pub_key);
+				DSA* tmpDSA=DSA_clone(m_pDSA);
 				tmpSig->m_pDSA=tmpDSA;
 			}
 		return tmpSig;
@@ -73,6 +72,29 @@ SINT32 CASignature::generateSignKey(UINT32 size)
 		return E_SUCCESS;
 	}
 
+SINT32 CASignature::getSignKey(DOM_DocumentFragment& node,DOM_Document& doc)
+	{
+		CACertificate* pCert;
+		getVerifyKey(&pCert);
+		EVP_PKEY* pPKey=EVP_PKEY_new();
+		EVP_PKEY_set1_DSA(pPKey,m_pDSA);		
+		PKCS12* pPKCS12=PKCS12_create(NULL,NULL, pPKey,pCert->m_pCert,NULL,0,0,0,0,0);
+		delete pCert;
+		EVP_PKEY_free(pPKey);
+		UINT8* buff=NULL;
+		SINT32 len=i2d_PKCS12(pPKCS12,&buff);
+		UINT32 outlen=2*len;
+		UINT8* outbuff=new UINT8[outlen];
+		CABase64::encode(buff,len,outbuff,&outlen);
+		outbuff[outlen]=0;
+		OPENSSL_free(buff);
+		node=doc.createDocumentFragment();
+		DOM_Element elem=doc.createElement("X509PKCS12");
+		node.appendChild(elem);
+		setDOMElementValue(elem,outbuff);
+		return E_SUCCESS;
+	}
+
 SINT32 CASignature::setSignKey(const DOM_Node& n,UINT32 type,const char* passwd)
 	{
 		DOM_Node node=n; 
@@ -114,20 +136,26 @@ SINT32 CASignature::setSignKey(const UINT8* buff,UINT32 len,UINT32 type,const ch
 					return parseSignKeyXML(buff,len);
 
 				case SIGKEY_PKCS12:
-					PKCS12* tmpPKCS12=d2i_PKCS12(NULL,(UINT8**)&buff,len);	
+					#if OPENSSL_VERSION_NUMBER	> 0x009070CfL
+						PKCS12* tmpPKCS12=d2i_PKCS12(NULL,(const UINT8**)&buff,len);	
+					#else
+						PKCS12* tmpPKCS12=d2i_PKCS12(NULL,(UINT8**)&buff,len);	
+					#endif
 					EVP_PKEY* key=NULL;
 //					X509* cert=NULL;
 					if(PKCS12_parse(tmpPKCS12,passwd,&key,NULL,NULL)!=1)
-						return E_UNKNOWN;
+							{
+								PKCS12_free(tmpPKCS12);
+								return E_UNKNOWN;
+							}	
+					PKCS12_free(tmpPKCS12);
 	//				X509_free(cert);
 					if(EVP_PKEY_type(key->type)!=EVP_PKEY_DSA)
 						{
 							EVP_PKEY_free(key);
 							return E_UNKNOWN;
 						}
-					DSA* tmpDSA=DSAparams_dup(key->pkey.dsa);
-					tmpDSA->priv_key=BN_dup(key->pkey.dsa->priv_key);
-					tmpDSA->pub_key=BN_dup(key->pkey.dsa->pub_key);
+					DSA* tmpDSA=DSA_clone(key->pkey.dsa);
 					EVP_PKEY_free(key);
 					if(DSA_sign_setup(tmpDSA,NULL,&tmpDSA->kinv,&tmpDSA->r)!=1)
 						{
@@ -218,6 +246,15 @@ SINT32 CASignature::parseSignKeyXML(const UINT8* buff,UINT32 len)
 	}
 
 
+SINT32 CASignature::sign(UINT8* in,UINT32 inlen,UINT8* sig,UINT32* siglen)
+	{
+		DSA_SIG* signature;
+		if(	sign(in,inlen,&signature)!=E_SUCCESS||
+				encodeRS(sig,siglen,signature)!=E_SUCCESS)
+			return E_UNKNOWN;
+		return E_SUCCESS;
+	}
+
 SINT32 CASignature::sign(UINT8* in,UINT32 inlen,DSA_SIG** pdsaSig)
 	{
 		UINT8 dgst[SHA_DIGEST_LENGTH];
@@ -239,7 +276,8 @@ SINT32 CASignature::getSignatureSize()
 	* @param out destination byte array which on return contains the XML Document including the XML Signature
 	* @param outlen size of destination byte array, on return contains the len of the signed XML document
 	* @param pIncludeCerts points to a CACertStore, which holds CACertificates, 
-	*					which should be included in the XML Signature for easy verification 
+	*					which should be included in the XML Signature for easy verification; if NULL 
+	*					no Certs will be included
 	*	@retval E_SUCCESS, if the Signature could be successful created
 	* @retval E_SPACE, if the destination byte array is to small for the signed XML Document
 	* @retval E_UNKNOWN, otherwise
@@ -265,7 +303,8 @@ SINT32 CASignature::signXML(UINT8* in,UINT32 inlen,UINT8* out,UINT32* outlen,CAC
 	* If ther is already a Signature is is removed first.
 	* @param node Node which should be signed 
 	* @param pIncludeCerts points to a CACertStore, which holds CACertificates, 
-	*					which should be included in the XML Signature for easy verification 
+	*					which should be included in the XML Signature for easy verification;
+	*					if null no certificates will be included
 	*	@retval E_SUCCESS, if the Signature could be successful created
 	* @retval E_UNKNOWN, otherwise
 */
@@ -318,6 +357,7 @@ SINT32 CASignature::signXML(DOM_Node& node,CACertStore* pIncludeCerts)
 		elemReference.appendChild(elemDigestValue);
 
 		// Signing the SignInfo block....
+
 		canonicalBuff=DOM_Output::makeCanonical(elemSignedInfo,&len);
 		if(canonicalBuff==NULL)
 			return E_UNKNOWN;
@@ -326,21 +366,24 @@ SINT32 CASignature::signXML(DOM_Node& node,CACertStore* pIncludeCerts)
 		
 		SINT32 ret=sign(canonicalBuff,len,&pdsaSig);
 		delete[] canonicalBuff;
+
 		if(ret!=E_SUCCESS)
 			{
 				DSA_SIG_free(pdsaSig);
 				return E_UNKNOWN;
 			}
-		memset(tmpBuff,0,40); //make first 40 bytes '0' --> if r or s is less then 20 bytes long! 
+		len=1024;
+		encodeRS(tmpBuff,&len,pdsaSig);
+		//memset(tmpBuff,0,40); //make first 40 bytes '0' --> if r or s is less then 20 bytes long! 
 													//(Due to be compatible to the standarad r and s must be 20 bytes each) 
-		BN_bn2bin(pdsaSig->r,tmpBuff+20-BN_num_bytes(pdsaSig->r)); //so r is 20 bytes with leading '0'...
-		BN_bn2bin(pdsaSig->s,tmpBuff+40-BN_num_bytes(pdsaSig->s));
+		//BN_bn2bin(pdsaSig->r,tmpBuff+20-BN_num_bytes(pdsaSig->r)); //so r is 20 bytes with leading '0'...
+		//BN_bn2bin(pdsaSig->s,tmpBuff+40-BN_num_bytes(pdsaSig->s));
 		
 		DSA_SIG_free(pdsaSig);
 
 		UINT sigSize=255;
 		UINT8 sig[255];
-		if(CABase64::encode(tmpBuff,40,sig,&sigSize)!=E_SUCCESS)
+		if(CABase64::encode(tmpBuff,len,sig,&sigSize)!=E_SUCCESS)
 			return E_UNKNOWN;
 		sig[sigSize]=0;
 
@@ -367,6 +410,48 @@ SINT32 CASignature::signXML(DOM_Node& node,CACertStore* pIncludeCerts)
 		return E_SUCCESS;
 	}
 
+
+SINT32 CASignature::getVerifyKey(CACertificate** ppCert)
+	{
+		//We need this DAS key as EVP key...
+		EVP_PKEY* pPKey=EVP_PKEY_new();
+		EVP_PKEY_set1_DSA(pPKey,m_pDSA);
+		*ppCert=new CACertificate();
+		(*ppCert)->m_pCert=X509_new();
+    // LERNGRUPPE 
+    // We nned to use Version 3 to use extensions
+// 		X509_set_version((*ppCert)->m_pCert,2);
+ 		X509_set_version((*ppCert)->m_pCert,3);
+		ASN1_TIME* pTime=ASN1_TIME_new();
+		ASN1_TIME_set(pTime,time(NULL));
+		X509_set_notBefore((*ppCert)->m_pCert,pTime);
+		X509_set_notAfter((*ppCert)->m_pCert,pTime);
+		X509_set_pubkey((*ppCert)->m_pCert,pPKey);
+// LERNGRUPPE 
+// Add the subjectKeyIdentifier-Extension to the certificate
+                if( (*ppCert)->setSubjectKeyIdentifier() != E_SUCCESS )
+                {
+                    CAMsg::printMsg( LOG_ERR, "Couldn't add the SKI to the certificate!\n");
+                    return E_UNKNOWN;
+                }
+
+		X509_sign((*ppCert)->m_pCert,pPKey,EVP_sha1());
+		EVP_PKEY_free(pPKey);
+		return E_SUCCESS;
+	}
+
+/** Calculates a SHA hash of the public key, which is represented as SubjectPublicKeyInfo
+	*/
+SINT32 CASignature::getVerifyKeyHash(UINT8* buff,UINT32* len)
+	{
+		 UINT8* tmpBuff=NULL;
+		 int l=i2d_DSA_PUBKEY(m_pDSA,&tmpBuff);
+		 SHA1(tmpBuff,l,buff);
+		 *len=SHA_DIGEST_LENGTH;
+		 OPENSSL_free(tmpBuff);
+		 return E_SUCCESS;
+	}
+
 /** Set the key for signature testing to the one include in pCert. If pCert ==NULL clears the
 	* signature test key
 	* @param pCert Certificate including the test key
@@ -387,25 +472,162 @@ SINT32 CASignature::setVerifyKey(CACertificate* pCert)
 				EVP_PKEY_free(key);
 				return E_UNKNOWN;
 			}
-		DSA* tmpDSA=DSAparams_dup(key->pkey.dsa);
-		tmpDSA->pub_key=BN_dup(key->pkey.dsa->pub_key);
+		DSA* tmpDSA=DSA_clone(key->pkey.dsa);
 		EVP_PKEY_free(key);
 		DSA_free(m_pDSA);
 		m_pDSA=tmpDSA;
 		return E_SUCCESS;
 	}
-/*
-SINT32 CASignature::verify(UINT8* in,UINT32 inlen,UINT8* sig,UINT32 siglen)
+	
+/**
+ * Parses the XML representation of a DSA public key
+ */
+SINT32 CASignature::setVerifyKey(const DOM_Element& xmlKey)
+{
+	UINT8 decodeBuffer[4096];
+	UINT32 len = 4096, encodedLen = 0;
+	DSA * tmpDSA = NULL;
+	
+	if(xmlKey==NULL)
 	{
-		if(m_pDSA==NULL)
-			return E_UNKNOWN;
-		UINT8 dgst[SHA_DIGEST_LENGTH];
-		SHA1(in,inlen,dgst);
-		if(DSA_verify(0,dgst,SHA_DIGEST_LENGTH,sig,siglen,m_pDSA)==1)
-		 return E_SUCCESS;
+		DSA_free(m_pDSA);
+		m_pDSA = NULL;
+		return E_SUCCESS;
+	}
+	char * tmpStr = xmlKey.getTagName().transcode();
+	if(strcmp(tmpStr, "JapPublicKey")!=0)
+	{
+		CAMsg::printMsg(LOG_DEBUG, 
+				"CASignature::setVerifyKey(): no JapPublicKey!"
+				"Tagname is %s", tmpStr);
+		delete[] tmpStr;
 		return E_UNKNOWN;
 	}
-*/
+	delete[] tmpStr;
+
+
+	if( getDOMElementAttribute(xmlKey,"version",decodeBuffer,&len)!=E_SUCCESS || 
+		strcmp((char*)decodeBuffer, "1.0")!=0 )
+	{
+		CAMsg::printMsg(LOG_DEBUG, 
+				"CASignature::setVerifyKey(): JapPublicKey has unknown version %s. "
+				"Version 1.0 expected!", tmpStr);
+		return E_UNKNOWN;
+	}
+	
+	DOM_Element elemDsaKey;
+	if(getDOMChildByName(xmlKey, (UINT8*)"DSAKeyValue", elemDsaKey, false)
+			!=E_SUCCESS)
+		{
+			CAMsg::printMsg(LOG_DEBUG, 
+					"CASignature::setVerifyKey(): DSAKeyValue not found!");
+			return E_UNKNOWN;
+		}
+	
+	tmpDSA=DSA_new();
+	
+	// parse "Y"
+	DOM_Element elem;
+	if(getDOMChildByName(elemDsaKey, (UINT8*)"Y", elem, false)
+			!=E_SUCCESS)
+		{
+			return E_UNKNOWN;
+		}
+	len=4096;
+	if(getDOMElementValue(elem, decodeBuffer, &len)!=E_SUCCESS)
+	{
+		DSA_free(tmpDSA);
+		return E_UNKNOWN;
+	}
+	encodedLen = len; len = 4096;
+	if(CABase64::decode(decodeBuffer, encodedLen, decodeBuffer, &len)!=E_SUCCESS)
+	{
+		DSA_free(tmpDSA);
+		return E_UNKNOWN;
+	}
+	tmpDSA->pub_key = BN_bin2bn(decodeBuffer,len,NULL);
+
+
+	// parse "G"
+	len = 4096;
+	if(getDOMChildByName(elemDsaKey, (UINT8*)"G", elem, false)
+			!=E_SUCCESS)
+		{
+			DSA_free(tmpDSA);
+			return E_UNKNOWN;
+		}
+	if(getDOMElementValue(elem, decodeBuffer, &len)!=E_SUCCESS)
+	{
+		DSA_free(tmpDSA);
+		return E_UNKNOWN;
+	}
+	encodedLen = len; len = 4096;
+	if(CABase64::decode(decodeBuffer, encodedLen, decodeBuffer, &len)!=E_SUCCESS)
+	{
+		DSA_free(tmpDSA);
+		return E_UNKNOWN;
+	}
+	tmpDSA->g=BN_bin2bn(decodeBuffer,len,NULL);
+		
+	
+	// parse "P"
+	len = 4096;
+	if(getDOMChildByName(elemDsaKey, (UINT8*)"P", elem, false)
+			!=E_SUCCESS)
+		{
+			DSA_free(tmpDSA);
+			return E_UNKNOWN;
+		}
+	if(getDOMElementValue(elem, decodeBuffer, &len)!=E_SUCCESS)
+	{
+		DSA_free(tmpDSA);
+		return E_UNKNOWN;
+	}
+	encodedLen = len; len = 4096;
+	if(CABase64::decode(decodeBuffer, encodedLen, decodeBuffer, &len)!=E_SUCCESS)
+	{
+		DSA_free(tmpDSA);
+		return E_UNKNOWN;
+	}
+	tmpDSA->p=BN_bin2bn(decodeBuffer,len,NULL);
+
+
+	// parse "Q"
+	len = 4096;
+	if(getDOMChildByName(elemDsaKey, (UINT8*)"Q", elem, false)
+			!=E_SUCCESS)
+		{
+			DSA_free(tmpDSA);
+			return E_UNKNOWN;
+		}
+	if(getDOMElementValue(elem, decodeBuffer, &len)!=E_SUCCESS)
+	{
+		DSA_free(tmpDSA);
+		return E_UNKNOWN;
+	}
+	encodedLen = len; 
+	len = 1024;
+	if(CABase64::decode(decodeBuffer, encodedLen, decodeBuffer, &len)!=E_SUCCESS)
+	{
+		DSA_free(tmpDSA);
+		return E_UNKNOWN;
+	}
+	tmpDSA->q=BN_bin2bn(decodeBuffer,len,NULL);
+	
+	if( tmpDSA->pub_key!=NULL && tmpDSA->g!=NULL && tmpDSA->p!=NULL && tmpDSA->q!=NULL)
+	{
+		if(m_pDSA!=NULL) 
+		{
+			DSA_free(m_pDSA);
+		}
+		m_pDSA = tmpDSA;
+		return E_SUCCESS;
+	}
+	DSA_free(tmpDSA);
+	return E_UNKNOWN;
+}
+
+
 SINT32 CASignature::verify(UINT8* in,UINT32 inlen,DSA_SIG* dsaSig)
 	{
 		if(m_pDSA==NULL||dsaSig==NULL||dsaSig->r==NULL||dsaSig->s==NULL)
@@ -417,6 +639,35 @@ SINT32 CASignature::verify(UINT8* in,UINT32 inlen,DSA_SIG* dsaSig)
 		return E_UNKNOWN;
 	}
 
+	
+/**
+ * Verifies an ASN.1 DER encoded SHA1-DSA signature
+ *
+ * @author Bastian Voigt
+ * @param in the document that was signed
+ * @param inlen, the document length
+ * @param dsaSig the DER encoded signature
+ * @param sigLen the signature length (normally 46 bytes)
+ * @return E_SUCCESS if the signature is valid, 
+ * 		E_UNKNOWN if an error occurs, 
+ * 		E_INVALID if the signature is invalid
+ */
+SINT32 CASignature::verifyDER(UINT8* in, UINT32 inlen, const UINT8 * dsaSig, const UINT32 sigLen)
+	{
+		UINT8 dgst[SHA_DIGEST_LENGTH];
+		UINT32 rc;
+
+		if(m_pDSA==NULL||dsaSig==NULL)
+			return E_UNKNOWN;
+		SHA1(in,inlen,dgst);
+		if((rc=DSA_verify(0, dgst, SHA_DIGEST_LENGTH, dsaSig, sigLen, m_pDSA))==1)
+			return E_SUCCESS;
+		else if(rc==0)
+			return E_INVALID; // wrong signature
+		return E_UNKNOWN;
+	}
+
+
 SINT32 CASignature::verifyXML(const UINT8* const in,UINT32 inlen)
 	{
 		MemBufInputSource oInput(in,inlen,"sigverify");
@@ -428,6 +679,7 @@ SINT32 CASignature::verifyXML(const UINT8* const in,UINT32 inlen)
 	}
 
 
+/** Verifies a XML Signature under node root.*/
 SINT32 CASignature::verifyXML(DOM_Node& root,CACertStore* trustedCerts)
 	{
 		DOM_Element elemSignature;
@@ -511,10 +763,10 @@ SINT32 CASignature::verifyXML(DOM_Node& root,CACertStore* trustedCerts)
 				return E_UNKNOWN;
 			}
 		DSA_SIG_free(dsaSig);
-				
-		root.removeChild(elemSignature);
+		DOM_Node tmpNode=root.removeChild(elemSignature);
 		outlen=5000;
 		DOM_Output::makeCanonical(root,out,&outlen);
+		root.appendChild(tmpNode);
 		UINT8 dgst1[SHA_DIGEST_LENGTH];
 		SHA1(out,outlen,dgst1);
 		delete[] out;
@@ -526,4 +778,30 @@ SINT32 CASignature::verifyXML(DOM_Node& root,CACertStore* trustedCerts)
 		return E_SUCCESS;
 	}
 
+SINT32 CASignature::encodeRS(UINT8* out,UINT32* outLen,DSA_SIG* pdsaSig)
+	{
+		UINT32 rSize, sSize;
+		memset(out,0,40); //make first 40 bytes '0' --> if r or s is less then 20 bytes long! 
+											//(Due to be compatible to the standarad r and s must be 20 bytes each) 
+		rSize = BN_num_bytes(pdsaSig->r);
+		sSize = BN_num_bytes(pdsaSig->s);
+		BN_bn2bin(pdsaSig->r,out+20-rSize); //so r is 20 bytes with leading '0'...
+		BN_bn2bin(pdsaSig->s,out+40-sSize);
+		*outLen=40;
+		return E_SUCCESS;
+	}
 
+	
+SINT32 CASignature::decodeRS(const UINT8* in, const UINT32 inLen, DSA_SIG* pDsaSig)
+{
+	ASSERT(pDsaSig!=NULL, "DSA_SIG is null");
+	ASSERT(inLen>20, "Inbuffer is <=20 bytes");
+	pDsaSig->r = BN_bin2bn(in, 20, NULL);
+	pDsaSig->s = BN_bin2bn(in+20, inLen-20, NULL);
+	return E_SUCCESS;
+}
+
+
+
+
+#endif //ONLY_LOCAL_PROXY
